use std::fs::File;
use crate::vec2::Vec2;

//todo: make dynamically different bitdepths
pub struct Image {
    data: Vec<u8>,
    size: Vec2<i32>
}

impl Image {
    pub fn new(size: Vec2<i32>) -> Self {
        Image {
            data: vec![0u8; (size.x * size.y) as usize],
            size
        }
    }

    pub fn load(path: &str) -> Self {
        let input = File::open(path).unwrap();
        let mut options = gif::DecodeOptions::new();
        options.set_color_output(gif::ColorOutput::Indexed);
        let mut decoder = options.read_info(input).unwrap();

        let x = decoder.width();
        let y = decoder.height();

        let mut data = vec![0u8; (x * y) as usize];
        decoder.next_frame_info().unwrap();
        decoder.read_into_buffer(&mut data).unwrap();
        Image {
            data,
            size: Vec2 {x: x as i32, y: y as i32}
        }
    }

    pub fn clear(&mut self) {
        self.fill(0);
    }

    pub fn data_mut(&mut self) -> &mut [u8] {
        self.data.as_mut_slice()
    }

    pub fn data(&mut self) -> &[u8] {
        self.data.as_slice()
    }

    pub fn draw_image(&mut self, pos: Vec2<i32>, image: &Image) {
        //todo: write proper implementation later
        for y in 0..image.size.y {
            for x in 0..image.size.x {
                //todo: implement better(very stupid to do per pixel)
                if x + pos.x >= 0 && y + pos.y >= 0 && x + pos.x < self.size.x && y + pos.y < self.size.y {
                    let p = image.get_pixel(Vec2 { x, y });
                    if p > 0 {
                        self.set_pixel(Vec2 { x: x + pos.x, y: y + pos.y }, p);
                    }
                }
            }
        }
    }

    pub fn draw_line(&mut self, pos1: Vec2<i32>, pos2: Vec2<i32>, color: u8) {
        let (x1, y1, x2, y2) = (pos1.x, pos1.y, pos2.x, pos2.y);

        let mut x = x1 as f32;
        let mut y = y1 as f32;
        let xdiff = (x2 - x1) as f32;
        let ydiff = (y2 - y1) as f32;
        let step = if xdiff.abs() > ydiff.abs() {
            xdiff.abs()
        } else {
            ydiff.abs()
        };
        let xs = xdiff / step;
        let ys = ydiff / step;
        for _ in 1..=(step as i32) {
            self.set_pixel(Vec2 { x: x as i32, y: y as i32 }, color);
            x += xs;
            y += ys;
        }
    }

    pub fn fill(&mut self, color: u8) {
        self.data.fill(color);
    }

    pub fn get_pixel(&self, pos: Vec2<i32>) -> u8 {
        self.data[(pos.x + self.size.x * pos.y) as usize]
    }

    pub fn set_pixel(&mut self, pos: Vec2<i32>, color: u8) {
        self.data[(pos.x + self.size.x * pos.y) as usize] = color;
    }
}
